function [ResEDF, ResUserData, Replace] = analyse(EDF, UserData, Opt)
% template for a plugin function
% a plugin *must* set the ResEDF values for:
%   Head.NS, Head.NRec, Head.PhysMin, Head.PhysMax, Labels

% Changes:
% 09/18/98 : data format changed
% (c) 2-Dec-1998 Michi Woertz
% (c) 3-Dec-1998 Alois Schloegl, modified. 
% (c) 1-Mar-1999 Alois Schloegl, major revision, PCA3 included, 
% (c) 2-Mar-1999 Alois Schloegl, ECG from PCA3 removed, sorting of EV by EW, PhysDim of PCA set to 1
% (c) 4-Mar-1999 Alois Schloegl, Regression analysis for removing HEOG, VEOG, REOG, and/or ECG

if nargin < 3
  Opt = '';
end;

Replace=0;         % a tribute to viewedf2

switch Opt
  case 'Reset'
    % Check whether EDF File has changed, update UserData, ...
    [ResEDF, ResUserData] = LocalReset(EDF, UserData);
  case 'Menu'
    % set UserData if necessary
    [ResEDF, ResUserData] = LocalConfigure(EDF, UserData);
  otherwise
    % Update ResEDF
    [ResEDF, ResUserData] = LocalUpdate(EDF, UserData);
end;
%return;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ResEDF, ResUserData] = LocalConfigure(EDF, UserData)    % channel selection only
if UserData.RESerror
   [ResEDF, ResUserData] = LocalReset(EDF, UserData);
   UserData=ResUserData;         % overcome the problem of recursions
   EDF=ResEDF;
   UserData.select=zeros(6,1);
else
   [UserData]=channels(UserData,2);
end;
[ResEDF, ResUserData] = LocalUpdate(EDF, UserData);
%return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ResEDF, ResUserData] = LocalReset(EDF, UserData)
% proove existence     % needed only once
FN=EDF.Head.FILE.Name;
UserData.RESerror=1;
ResUserData=UserData;
ResEDF=EDF;
ResEDF.Head.NS=0;
UserData.select=zeros(6,1);

m=[8 1 1; 9 1 -1; 1 2 1; 4 2 1; 7 2 1; 8 2 -1; 9 2 -1; 3 3 1; 6 3 1; 7 3 2; 8 3 -2; 9 3 -2; 12 4 1];
m=sparse(m(:,1),m(:,2),m(:,3),EDF.Head.NS,4);
m(:,2)=m(:,2)/2;
m(:,3)=m(:,3)/4;

clear IN XC310 RES

%cmd=['load (''' FN 'res.mat'', ''RES'' ) ' ]; eval(cmd,'-1;');
cmd=['load ' FN 'res.mat RES']; eval(cmd,'-1;');

if 0; ~(exist('RES')==1)
        [FILENAME, PATHNAME] = uigetfile([ FN 'res.mat'],['Searching the ' FN 'in.mat-file...']);
        cmd=['load ' PATHNAME, FN 'res.mat RES']; eval(cmd,'-1;');
        if ~(exist('RES')==1)
                fprintf(2,'Error in plug-in %s: %sres.mat invalid or not found\n',mfilename,FN);
                %return;
        end
end;

%eval('load([FN ''cov310a.mat'']);', '-1;');
cmd=['load ' FN 'cov310a.mat XC310']; eval(cmd,'-1;');

if 0; ~(exist('XC310')==1)
        [FILENAME, PATHNAME] = uigetfile([ FN 'cov310a.mat'],['Searching the ' FN 'in.mat-file...']);
        cmd=['load ' PATHNAME, FN 'cov310a.mat XC310']; eval(cmd,'-1;');
        if ~(exist('XC310')==1)
                fprintf(2,'Error in plug-in %s: %scov310a.mat invalid or not found\n',mfilename,FN);
                %return;
        end
end;

UserData.RESerror=0;
UserData.FN=FN;

if FN(1)=='n' 
        IN.Rchansel=[1:9 17:21 12];
else
        IN.Rchansel=[1:9 12];
end;

%IN.chansel=[1:9 12];
fprintf(1,'PreProc REG3:\n');
if isfield(RES,'XCN')
        %IN.Rchansel=[1:9 12];
        %M=m(IN.Rchansel,:)
        IN.XCN=RES.XCN;
        IN.REG3m=m;
        %IN.REG1b=(eye(length(IN.Rchansel))-M*((M'*cov*M)\(M'*cov)));
        fprintf(1,'REG3: OK\n');
else
        fprintf(2,'REG3 not available in %s\n',FN);
        warnh = warndlg('REG3 not available for this file','regr3 Warning');
end;


UserData.IN=IN; 
UserData=channels(UserData,2);    % channel selection
[ResEDF, ResUserData] = LocalUpdate(EDF, UserData);
%return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ResEDF, ResUserData] = LocalUpdate(EDF, UserData)
% This function returns the first channel of the first EDF file
if ~UserData.RESerror
   [ResEDF,ResUserData]=eogtmpl(EDF, UserData);
else
   ResEDF=EDF;                 % no valid RES-file present, nothing can be calculated
   ResUserData=UserData;
   ResEDF.Head.NS=0;
end;
%return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	Version 0.70
%	20.08.1998
%	Copyright (c) by  Alois Schloegl
%	a.schloegl@ieee.org	
% Init
function [ResEDF,ResUserData]=eogtmpl(EDF, UserData);

% Selektiert die Kanle EEG, EOG und ECG [1:9 12]   
% belasse ich derweil so, Rechnung scheint schnell genug... MW
% ersetze aber die Kanalauswahl fr das Display...

IN=UserData.IN;                        % do not reload res-Data
maxspr=0;
%maxspr=max(EDF.Head.SPR);
for k=1:EDF.Head.NS,%K=IN.Rchansel(k);
        maxspr=max(maxspr,length(EDF.Record{k}));
end;                                   % UserData required for the Display-Option
S=zeros(maxspr,EDF.Head.NS);
for k=1:EDF.Head.NS,%length(IN.chansel),K=IN.chansel(k);
        K=k;
        if maxspr==length(EDF.Record{K})
                tmp=EDF.Record{K}(:);
        else
                tmp=reshape((ones(maxspr/length(EDF.Record{K}),1)*(EDF.Record{K})'),maxspr,1);
        end;
        S(:,k)=tmp;
end;                                   % UserData required for the Display-Option

ResEDF=EDF;
ResUserData=UserData;
ResEDF.Head.NS=0;                      % up to now
count=1;                               % counter for calculated channels


if 1
        %M=IN.REG3m(:,find(UserData.select));
        sel=find(UserData.select);
        M=IN.REG3m(IN.Rchansel,sel);
        [mu,sd,COV,xc]=decovm(IN.XCN);
        cov=COV(IN.Rchansel,IN.Rchansel);
        IN.REG3b=(eye(length(IN.Rchansel))-M*((M'*cov*M)\(M'*cov)));
else
        if UserData.select(1)
        end;
        if UserData.select(2)
        end;
        if UserData.select(3)
        end;
        if UserData.select(4)
        end;
end;

NS=size(IN.REG3m,2);
%disp([IN.REG1m IN.REG1b])
REG1.out=S(:,IN.Rchansel)*IN.REG3m(IN.Rchansel,:);
REG1.Label=[(UserData.select(1:4)*13+32), [' RegR3: HEOG   ';' RegR3: VEOG   ';' RegR3: REOG   '; ' RegR3: ECG    ']];
ResEDF.Head.Label(count+(0:NS-1),:)=REG1.Label(1:NS,:);
ResEDF.Record(count+(0:NS-1))=num2cell(REG1.out,1);    % put into output
tmp=[8 8 8 12];
ResEDF.Head.PhysDim(count+(0:NS-1),:)=EDF.Head.PhysDim(tmp,:);
ResEDF.Head.PhysMin(count:NS+count-1)=EDF.Head.PhysMin(tmp);
ResEDF.Head.PhysMax(count:NS+count-1)=EDF.Head.PhysMax(tmp);
ResEDF.Head.NRec=EDF.Head.NRec;
count=count+NS; 


NS=size([IN.REG3b],2);
%disp([IN.REG1m IN.REG1b])
REG3.out=S(:,IN.Rchansel)*[IN.REG3b];
REG3.Label=[[ones(NS,1)*'Reg3C: ' EDF.Head.Label(IN.Rchansel,1:9)]];
ResEDF.Head.Label(count+(0:NS-1),:)=REG3.Label(1:NS,:);
ResEDF.Record(count+(0:NS-1))=num2cell(REG3.out,1);    % put into output
ResEDF.Head.PhysDim(count+(0:NS-1),:)=EDF.Head.PhysDim([IN.Rchansel],:);
ResEDF.Head.PhysMin(count:NS+count-1)=EDF.Head.PhysMin([IN.Rchansel]);
ResEDF.Head.PhysMax(count:NS+count-1)=EDF.Head.PhysMax([IN.Rchansel]);
ResEDF.Head.NRec=EDF.Head.NRec;
count=count+NS; 

ResEDF.Head.NS=count-1;                % total number of calculated signals...

%save tmp.mat S EDF PCA1 
% didn't know what to fill in - must be applied in each if-block differently???

% nein, muss nicht bei jedem Block neu berechnet werden.
% die Calibirierung ist immer 1, der Offset immer 0
% Ebenfalls ist es nicht notwendig die Labels neu zu berechnen
% Dies ist nur notwendig wenn man die Auswahl aendert.
% Daher sollte - anstatt bei jeder neuen Anzeige - diese Berechnung 
% sowie die Labels beim Auswahlmenue berechnet werden. 

%statt
%ResEDF.Head.NRec=EDF.Head.NRec(1);
%ResEDF.Head.PhysMin(1:count)=EDF.Head.PhysMin(1);
%ResEDF.Head.PhysMax(1:count)=EDF.Head.PhysMax(1);
%besser

%return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LocalSelectChannels
% select channels to be displayed on the screen
function [UserDaten]=channels(UserDaten,what)

% what=1: select all
% what=0: unselect all
% what=2: no change

chans=1:5;
if what~=2
   UserDaten.select(chans)=what;     % pre-set
end;
Data = get(gcf, 'UserData');
if isempty(Data.EDF)
  errordlg('No EDF-File is open!', 'Error');
  return;
end
% find selected channels
text=['Which components should be removed ...'];
dlgh = dialog(...
    'Name', text, ...
    'CloseRequestFcn', 'set(gcf,''UserData'',''Cancel'');uiresume;');
cbx = 0.1;
cby = 0.95;
cbxspace = 0.4;
cbyspace = 0.8 / 2 + 0.005; 
cbwidth = 0.35;
cbheight = LocalGetFontHeight;
% display EDF channels
text=['HEOG';'VEOG';'REOG';'ECG '];
for i = 1:4;
  uih(i) = uicontrol(dlgh, ...
    'Style', 'CheckBox', ...
    'Units', 'Normalized', ...
    'String', sprintf('Mode : %s', text(i, :)), ...
    'Value', UserDaten.select(i), ...
    'Position', [cbx + rem(i-1,2)*cbxspace, cby - floor((i-1)/2)*cbyspace, ...
       cbwidth, cbheight]);
end;
uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'Cancel', ...
    'Position', [0.1, 0.02, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''Cancel'');uiresume;');
uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'OK', ...
    'Position', [0.7, 0.02, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''OK'');uiresume;');
 uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'Unselect', ...
    'Position', [0.1, 0.08, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''Unselect'');uiresume;');
 uicontrol(dlgh, ...
    'Style', 'PushButton', ...
    'Units', 'Normalized', ...
    'String', 'Select', ...
    'Position', [0.7, 0.08, 0.2, 0.05], ...
    'Callback', 'set(gcf,''UserData'',''Select'');uiresume;');
uiwait(dlgh);
changed = 0;
hilfe=get(dlgh,'UserData');
if strcmp(hilfe,'OK')
   changed = 1;
   for i=1:4;   UserDaten.select(i)=get(uih(i),'Value');    end;
end; 
if strcmp(hilfe,'Unselect')
   delete(dlgh);
   [UserDaten]=channels(UserDaten,0);   %unselect all
   return;
elseif strcmp(hilfe,'Select')
   delete(dlgh);
   [UserDaten]=channels(UserDaten,1);   %select all
   return;
end;
delete(dlgh);
if ~(sum(UserDaten.select))               
   text=['No calculation selected!'];
   h=errordlg(text, 'Warning');
   waitfor(h);
end;
%return;

% LocalGetFontHeight
% get Fontheight
function hght=LocalGetFontHeight()
tempH = uicontrol(...
    'Style', 'Text', ...
    'String', 'Gg', ...
    'Units', 'Normalized', ...
    'FontUnits', 'Normalized', ...
    'Position', [0, 0, 1, 1], ...
    'Visible', 'off');
hght = get(tempH, 'FontSize') * 1.25; % 1.25 makes things look better
delete(tempH);
%return;

